library(igraph)						
library(abind)						
library(tcltk2)	
library(raster)
library(rgdal)

#setwd("z:/Salmn/Bidasoa")

#############################################################################
#############Create geometric network from shapefile river layer#############
#############################################################################
#hqi<-read.table("Index_after.txt",header=T)
#p<-length(bidasoa)
#bidasoa$hqi<-1
#for (i in 1:p){
#  for (j in 1:p){
#    if (bidasoa$ID[i]==hqi$FID[j]) {
#      bidasoa$hqi[i]<-hqi$Index[j]
#    }
#  }
#}

bidasoa <- shapefile("Bidasoa1992Nat_OK.shp")
bidasoa$ID <- bidasoa$ID <- c(0:(length(bidasoa)-1))
c <- t(sapply(unlist(coordinates(bidasoa), recursive=FALSE), FUN=function (x) cbind(x[1, ], x[nrow(x), ])))
c <- as.data.frame(c)
names(c) <- c("From.x", "From.y", "To.x", "To.y")
bidasoa.table<-as.data.frame(bidasoa)
n2 <- cbind(bidasoa.table, c)
v <- unique(rbind(data.frame(X=n2$From.x, Y=n2$From.y), data.frame(X=n2$To.x, Y=n2$To.y)))
v$ID <- 1:nrow(v)
n3 <- merge(n2, data.frame(From=v$ID, From.x=v$X, From.y=v$Y))
n4 <- merge(n3, data.frame(To=v$ID, To.x=v$X, To.y=v$Y))
edge.list <-cbind(n4[ ,c("From", "To")], n4[ ,grep("^(From|To)$", names(n4), invert=TRUE)])
edge.list <- edge.list[ ,c(2, 1, 3:23)]
#edge.list$hq<-edge.list$GEOM_LONG*edge.list$hqi
#bidasoa$hq<-bidasoa$GEOM_LONG*bidasoa$hqi
names(edge.list)[1] <- "From"
names(edge.list)[2] <- "To"
g <- graph.data.frame(edge.list, directed=T) #Create graph 

#Plot graph
tkplot(g)

####Connectivity matrix (0, 1)
gadj <- as_adjacency_matrix(g, sparse=F)

####Create dams dataframe
dams <- shapefile("Presas2016Nat_updown.shp")
dams.data.frame <- as.data.frame(dams)
dams.data.frame$ID<-c(1:length(dams.data.frame$NOMBRE))
#dams.data.frame$Pasab_Des<-as.numeric(dams.data.frame$Pasab_Des)
#dams.data.frame$updown<-dams.data.frame$Pasabilida*dams.data.frame$Pasab_Des

##Small permeable obstacles are considered passable
dams.data.frame$Pasabilida[15]<-1
dams.data.frame$Pasabilida[32]<-1
dams.data.frame$Pasabilida[70]<-1
dams.data.frame$Pasabilida[77]<-1
dams.data.frame$Pasabilida[94]<-1

#########################################
########Connectivity Calculations########
#########################################
connectivity <- function(n, m.nput, m.cput, length, L){
  #Compute cumulative upstream passage
  paths.u <- array(m.cput, dim = c(n,n,n))			#Create an empty 3D array to store path lengths
  
  Futemp <- m.cput						#Initialize path length of 1
  for (k in 2:n){						#Loop over each potential path length
    Futemp <- Futemp %*% m.cput
    paths.u[ , ,k] <- Futemp
  }
  
  #Compute cumulative passage rates to the specified node
  cpu <- matrix(0, nrow = n, ncol = 1)				#Create an empty matrix for cumulative passage rates
  for (k in 1:n){cpu[k] <- max(paths.u[k, 1, ])}
  cpu[1] <- 1						#Set passage into the network 
  
  #Compute the total passage rate beyond the node
  npu <- matrix(0, nrow = n, ncol = 1)				#Create an empty matrix for nodal passage rates
  for (i in 1:n){npu[i] <- max(m.nput[i, ])}		#Nodal passage rate
  npu[1] <- 1
  tpu <- cpu * npu					#Total passage rate
  
  
  #Compute network connectivity based on the upstream passage to a point and the associated downstream habitat
  Cu <- sum(length*tpu)					#Compute length of accessible upstream habitat
  Cunorm <- Cu/L						#Normalize C by the total habitat length
  
  #Send outputs
  Cunorm
}

####Create dataframe to store results
analisis <- data.frame(PRESA=as.numeric(), CONECTIVIDAD=as.numeric())

########################
########Function########
########################

for (dd in 1:nrow(dams.data.frame)) {
  dams.data.frame <- as.data.frame(dams)
  dams.data.frame <- dams.data.frame[-dd, ]
  
  ####Create node dataframe and passability matrix
  ##For npu
  nodes <- data.frame(edge.list$From, edge.list$To, edge.list$From.x, edge.list$From.y)
  names(nodes)[3] <- "x"
  names(nodes)[4] <- "y"
  
  m <- length(nodes$edge.list.From)   
  o <- length(dams.data.frame$NOMBRE)   
  nodes$pas <- 1
  for (i in 1:m){
    for (j in 1:o){
      if (round(nodes$x[i], 2) == round(dams.data.frame$coords.x1[j], 2)) {
        nodes$pas[i] <- dams.data.frame$Pasabilida[j]
      }
    }
  }
  
  summary(nodes$pas < 1)    ##To check
  
  m.npu <- gadj
  for (i in 1:m) {
    m.npu[as.character(nodes[i,1]), as.character(nodes[i,2])] = nodes$pas[i]
  }
  
  #Transpose matrix to make first row the first node 
  m.nput <- t(m.npu)
  
  m.nput <- m.nput[c(29, 1, c(2:28,30:nrow(m.nput))), ]
  m.nput <- m.nput[ ,c(29, 1, c(2:28,30:ncol(m.nput)))]
  
  ##For cpu
  nodes2 <- data.frame(edge.list$From, edge.list$To, edge.list$To.x, edge.list$To.y)
  names(nodes2)[3] <- "x"
  names(nodes2)[4] <- "y"
  
  nodes2$pas <- 1
  for (i in 1:m){
    for (j in 1:o){
      if (round(nodes2$x[i], 2) == round(dams.data.frame$coords.x1[j], 2)) {
        nodes2$pas[i] <- dams.data.frame$Pasabilida[j]
      }
    }
  }
  
  summary(nodes2$pas < 1) #To check
  
  m.cpu <- gadj
  for (i in 1:m) {
    m.cpu[as.character(nodes2[i,1]), as.character(nodes2[i,2])] = nodes2$pas[i]
  }
  
  #Transpose matrix and put first node in firs row
  m.cput <- t(m.cpu)
  m.cput <- m.cput[c(29, 1, c(2:28,30:nrow(m.cput))), ]
  m.cput <- m.cput[, c(29, 1, c(2:28,30:ncol(m.cput)))]
  
  ####Create dataframe with segment length
  l <- data.frame(edge.list$From, edge.list$To, edge.list$From.x, edge.list$From.y, edge.list$GEOM_LONG)
  
  ##Replace nodes with segment lenght in matrix
  m.length <- gadj
  
  for (i in 1:m) {
    m.length[as.character(l[i,1]), as.character(l[i,2])] = l$edge.list.GEOM_LONG[i]
  }
  
  ##Transpose matrix
  m.lengtht <- t(m.length)
  m.lengtht <- m.lengtht[c(29, 1,c(2:28, 30:nrow(m.lengtht))), ]
  m.lengtht <- m.lengtht[, c(29, 1,c(2:28, 30:ncol(m.lengtht)))]
  
  ##Compress matrix with segment length sum
  n <- m + 1
  
  ##Create segment lengths
  length <- matrix(0, nrow=n, ncol=1)	#Creeate matrix to store data	
  for (i in 1:n){
    length[i] <- sum(m.lengtht[ ,i])
  } 
  
  ####Variables
  #n
  #m.nput
  #m.cput
  #length
  L <- sum(length)
  
  ####Run function 
  Cunorm <- connectivity(n, m.nput, m.cput, length, L)
  
  ####Join results to dataframe
  analisis <- rbind(analisis, c(dd, Cunorm))
 
}

####Add dam name column
analisis$Nombre_X1 <- 1
for(a in 1:nrow(dams.data.frame)){
  for(b in 1:nrow(analisis)){
    if (dams.data.frame$ID[a] == analisis$X1[b]){
      analisis$Nombre_X1[b] <- dams.data.frame$NOMBRE[a]
    }
  }
}

####Export data to .txt file
write.table(analisis,"D:/Salmn/Bidasoa/Presa1.txt",sep="\t")


